@@ -965,7 +965,8 @@ cc_display_monitor_dbus_new (GVariant *variant,
 typedef enum _CcDisplayLayoutMode
 {
   CC_DISPLAY_LAYOUT_MODE_LOGICAL = 1,
-  CC_DISPLAY_LAYOUT_MODE_PHYSICAL = 2
+  CC_DISPLAY_LAYOUT_MODE_PHYSICAL = 2,
+  CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL = 3
 } CcDisplayLayoutMode;
 
 typedef enum _CcDisplayConfigMethod
@@ -993,6 +994,8 @@ struct _CcDisplayConfigDBus
   gboolean supports_changing_layout_mode;
   gboolean global_scale_required;
   CcDisplayLayoutMode layout_mode;
+  gint legacy_ui_scale;
+  char *renderer;
 
   GList *monitors;
   CcDisplayMonitorDBus *primary;
@@ -1159,6 +1162,9 @@ cc_display_config_dbus_equal (CcDisplayConfig *pself,
   g_return_val_if_fail (pself, FALSE);
   g_return_val_if_fail (pother, FALSE);
 
+  if (self->layout_mode != other->layout_mode)
+    return FALSE;
+
   cc_display_config_dbus_ensure_non_offset_coords (self);
   cc_display_config_dbus_ensure_non_offset_coords (other);
 
@@ -1265,8 +1271,12 @@ cc_display_config_dbus_set_cloning (CcDisplayConfig *pself,
     {
       logical_monitor = g_object_new (CC_TYPE_DISPLAY_LOGICAL_MONITOR, NULL);
       for (l = self->monitors; l != NULL; l = l->next)
-        cc_display_monitor_dbus_set_logical_monitor (CC_DISPLAY_MONITOR_DBUS (l->data),
-                                                     logical_monitor);
+        {
+          if (!cc_display_monitor_is_usable (l->data))
+            continue;
+          cc_display_monitor_dbus_set_logical_monitor (CC_DISPLAY_MONITOR_DBUS (l->data),
+                                                       logical_monitor);
+        }
       register_logical_monitor (self, logical_monitor);
     }
   else if (!clone && is_cloning)
@@ -1434,7 +1444,8 @@ cc_display_config_dbus_generate_cloning_modes (CcDisplayConfig *pself)
         best_mode = virtual_mode;
     }
 
-  best_mode->flags |= MODE_PREFERRED;
+  if (best_mode)
+    best_mode->flags |= MODE_PREFERRED;
 
   return clone_modes;
 }
@@ -1453,7 +1464,52 @@ cc_display_config_dbus_is_layout_logical (CcDisplayConfig *pself)
 {
   CcDisplayConfigDBus *self = CC_DISPLAY_CONFIG_DBUS (pself);
 
-  return self->layout_mode == CC_DISPLAY_LAYOUT_MODE_LOGICAL;
+  return self->layout_mode == CC_DISPLAY_LAYOUT_MODE_LOGICAL ||
+         self->layout_mode == CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL;
+}
+
+static void
+cc_display_config_dbus_set_layout_logical (CcDisplayConfig *pself,
+                                           gboolean         logical)
+{
+  CcDisplayConfigDBus *self = CC_DISPLAY_CONFIG_DBUS (pself);
+
+  if (!self->supports_changing_layout_mode)
+    return;
+
+  if (!logical)
+    {
+      self->layout_mode = CC_DISPLAY_LAYOUT_MODE_PHYSICAL;
+      return;
+    }
+
+  if (g_str_equal (self->renderer, "native") || g_str_equal (self->renderer, "kms"))
+    self->layout_mode = CC_DISPLAY_LAYOUT_MODE_LOGICAL;
+  else if (g_str_equal (self->renderer, "xrandr"))
+    self->layout_mode = CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL;
+  else
+    g_return_if_reached ();
+}
+
+static gboolean
+cc_display_config_dbus_layout_use_ui_scale (CcDisplayConfig *pself)
+{
+  CcDisplayConfigDBus *self = CC_DISPLAY_CONFIG_DBUS (pself);
+  return self->layout_mode == CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL;
+}
+
+static gint
+cc_display_config_dbus_get_legacy_ui_scale (CcDisplayConfig *pself)
+{
+  CcDisplayConfigDBus *self = CC_DISPLAY_CONFIG_DBUS (pself);
+  return self->legacy_ui_scale;
+}
+
+static const char *
+cc_display_config_dbus_get_renderer (CcDisplayConfig *pself)
+{
+  CcDisplayConfigDBus *self = CC_DISPLAY_CONFIG_DBUS (pself);
+  return self->renderer;
 }
 
 static gboolean
@@ -1623,6 +1679,11 @@ cc_display_config_dbus_init (CcDisplayConfigDBus *self)
   self->global_scale_required = FALSE;
   self->layout_mode = CC_DISPLAY_LAYOUT_MODE_LOGICAL;
   self->logical_monitors = g_hash_table_new (NULL, NULL);
+
+  if (g_getenv ("WAYLAND_DISPLAY"))
+    self->renderer = g_strdup ("native");
+  else if (g_getenv ("DISPLAY"))
+    self->renderer = g_strdup ("xrandr");
 }
 
 static void
@@ -1680,6 +1741,8 @@ construct_monitors (CcDisplayConfigDBus *self,
                                  self, G_CONNECT_SWAPPED);
     }
 
+  self->monitors = g_list_reverse (self->monitors);
+
   while (TRUE)
     {
       g_autoptr(GVariant) variant = NULL;
@@ -1758,10 +1821,9 @@ update_panel_orientation_managed (CcDisplayConfigDBus *self)
 }
 
 static void
-proxy_properties_changed_cb (GDBusProxy          *proxy,
+proxy_properties_changed_cb (CcDisplayConfigDBus *self,
                              GVariant            *changed_properties,
-                             GStrv                invalidated_properties,
-                             CcDisplayConfigDBus *self)
+                             GStrv                invalidated_properties)
 {
   GVariantDict dict;
 
@@ -1807,12 +1869,21 @@ cc_display_config_dbus_constructed (GObject *object)
         {
           g_variant_get (v, "b", &self->global_scale_required);
         }
+      else if (g_str_equal (s, "legacy-ui-scaling-factor"))
+        {
+          g_variant_get (v, "i", &self->legacy_ui_scale);
+        }
+      else if (g_str_equal (s, "renderer"))
+        {
+          g_clear_pointer (&self->renderer, g_free);
+          g_variant_get (v, "s", &self->renderer);
+        }
       else if (g_str_equal (s, "layout-mode"))
         {
           guint32 u = 0;
           g_variant_get (v, "u", &u);
           if (u >= CC_DISPLAY_LAYOUT_MODE_LOGICAL &&
-              u <= CC_DISPLAY_LAYOUT_MODE_PHYSICAL)
+              u <= CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL)
             self->layout_mode = u;
         }
     }
@@ -1831,8 +1902,8 @@ cc_display_config_dbus_constructed (GObject *object)
   if (error)
     g_warning ("Could not create DisplayConfig proxy: %s", error->message);
 
-  g_signal_connect (self->proxy, "g-properties-changed",
-                    G_CALLBACK (proxy_properties_changed_cb), self);
+  g_signal_connect_swapped (self->proxy, "g-properties-changed",
+                            G_CALLBACK (proxy_properties_changed_cb), self);
   update_panel_orientation_managed (self);
 
   G_OBJECT_CLASS (cc_display_config_dbus_parent_class)->constructed (object);
@@ -1910,6 +1981,7 @@ cc_display_config_dbus_finalize (GObject *object)
 
   g_clear_list (&self->monitors, g_object_unref);
   g_clear_pointer (&self->logical_monitors, g_hash_table_destroy);
+  g_clear_pointer (&self->renderer, g_free);
 
   G_OBJECT_CLASS (cc_display_config_dbus_parent_class)->finalize (object);
 }
@@ -1935,10 +2007,14 @@ cc_display_config_dbus_class_init (CcDisplayConfigDBusClass *klass)
   parent_class->set_cloning = cc_display_config_dbus_set_cloning;
   parent_class->generate_cloning_modes = cc_display_config_dbus_generate_cloning_modes;
   parent_class->is_layout_logical = cc_display_config_dbus_is_layout_logical;
+  parent_class->set_layout_logical = cc_display_config_dbus_set_layout_logical;
   parent_class->is_scaled_mode_valid = cc_display_config_dbus_is_scaled_mode_valid;
   parent_class->set_minimum_size = cc_display_config_dbus_set_minimum_size;
   parent_class->get_panel_orientation_managed =
     cc_display_config_dbus_get_panel_orientation_managed;
+  parent_class->layout_use_ui_scale = cc_display_config_dbus_layout_use_ui_scale;
+  parent_class->get_legacy_ui_scale = cc_display_config_dbus_get_legacy_ui_scale;
+  parent_class->get_renderer = cc_display_config_dbus_get_renderer;
 
   pspec = g_param_spec_variant ("state",
                                 "GVariant",
@@ -1999,6 +2075,26 @@ logical_monitor_is_rotated (CcDisplayLogicalMonitor *lm)
     }
 }
 
+static double
+get_maximum_scale (CcDisplayConfig *config)
+{
+  GList *outputs, *l;
+  double max_scale = 1.0;
+  outputs = cc_display_config_get_monitors (config);
+
+  for (l = outputs; l; l = l->next)
+    {
+      CcDisplayMonitor *output = l->data;
+
+      if (!cc_display_monitor_is_useful (output))
+        continue;
+
+      max_scale = MAX (max_scale, cc_display_monitor_get_scale (output));
+    }
+
+  return max_scale;
+}
+
 static int
 logical_monitor_width (CcDisplayLogicalMonitor *lm)
 {
@@ -2017,6 +2113,11 @@ logical_monitor_width (CcDisplayLogicalMonitor *lm)
 
   if (monitor->config->layout_mode == CC_DISPLAY_LAYOUT_MODE_LOGICAL)
     return round (width / lm->scale);
+  if (monitor->config->layout_mode == CC_DISPLAY_LAYOUT_MODE_GLOBAL_UI_LOGICAL)
+    {
+      double max_scale = get_maximum_scale(CC_DISPLAY_CONFIG (monitor->config));
+      return round ((width * ceil (max_scale)) / lm->scale);
+    }
   else
     return width;
 }
