@@ -41,11 +41,11 @@
 #include "pp-printer.h"
 
 struct _PpOptionsDialog {
-  GtkDialog    parent_instance;
+  AdwWindow    parent_instance;
 
   GtkTreeSelection *categories_selection;
   GtkTreeView      *categories_treeview;
-  GtkBox           *main_box;
+  GtkWidget        *main_box;
   GtkNotebook      *notebook;
   GtkSpinner       *spinner;
   GtkStack         *stack;
@@ -64,7 +64,7 @@ struct _PpOptionsDialog {
   gboolean sensitive;
 };
 
-G_DEFINE_TYPE (PpOptionsDialog, pp_options_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (PpOptionsDialog, pp_options_dialog, ADW_TYPE_WINDOW)
 
 enum
 {
@@ -295,44 +295,31 @@ ppd_option_name_translate (ppd_option_t *option)
 static gint
 grid_get_height (GtkWidget *grid)
 {
-  GList *children;
-  GList *child;
+  GtkWidget *child;
   gint   height = 0;
-  gint   top_attach = 0;
+  gint   row = 0;
   gint   max = 0;
 
-  children = gtk_container_get_children (GTK_CONTAINER (grid));
-  for (child = children; child; child = g_list_next (child))
+  for (child = gtk_widget_get_first_child (grid);
+       child;
+       child = gtk_widget_get_next_sibling (child))
     {
-      gtk_container_child_get (GTK_CONTAINER (grid), child->data,
-                               "top-attach", &top_attach,
-                               "height", &height,
-                               NULL);
+      gtk_grid_query_child (GTK_GRID (grid),
+                            child,
+                            NULL, &row,
+                            NULL, &height);
 
-      if (height + top_attach > max)
-        max = height + top_attach;
+      if (height + row > max)
+        max = height + row;
     }
 
-  g_list_free (children);
-
   return max;
 }
 
 static gboolean
 grid_is_empty (GtkWidget *grid)
 {
-  GList *children;
-
-  children = gtk_container_get_children (GTK_CONTAINER (grid));
-  if (children)
-    {
-      g_list_free (children);
-      return FALSE;
-    }
-  else
-    {
-      return TRUE;
-    }
+  return gtk_widget_get_first_child (grid) == NULL;
 }
 
 static GtkWidget *
@@ -344,7 +331,6 @@ ipp_option_add (IPPAttribute *attr_supported,
                 GtkWidget    *grid,
                 gboolean      sensitive)
 {
-  GtkStyleContext *context;
   GtkWidget       *widget;
   GtkWidget       *label;
   gint             position;
@@ -355,15 +341,13 @@ ipp_option_add (IPPAttribute *attr_supported,
                                                    printer_name);
   if (widget)
     {
-      gtk_widget_show (widget);
+      gtk_widget_set_visible (widget, TRUE);
       gtk_widget_set_sensitive (widget, sensitive);
       position = grid_get_height (grid);
 
       label = gtk_label_new (option_display_name);
-      gtk_widget_show (GTK_WIDGET (label));
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-      context = gtk_widget_get_style_context (label);
-      gtk_style_context_add_class (context, "dim-label");
+      gtk_widget_add_css_class (label, "dim-label");
       gtk_widget_set_halign (label, GTK_ALIGN_END);
       gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
       gtk_widget_set_margin_start (label, 10);
@@ -382,7 +366,6 @@ ppd_option_add (ppd_option_t  option,
                 GtkWidget    *grid,
                 gboolean      sensitive)
 {
-  GtkStyleContext *context;
   GtkWidget       *widget;
   GtkWidget       *label;
   gint             position;
@@ -390,15 +373,13 @@ ppd_option_add (ppd_option_t  option,
   widget = (GtkWidget *) pp_ppd_option_widget_new (&option, printer_name);
   if (widget)
     {
-      gtk_widget_show (widget);
+      gtk_widget_set_visible (widget, TRUE);
       gtk_widget_set_sensitive (widget, sensitive);
       position = grid_get_height (grid);
 
       label = gtk_label_new (ppd_option_name_translate (&option));
-      gtk_widget_show (GTK_WIDGET (label));
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-      context = gtk_widget_get_style_context (label);
-      gtk_style_context_add_class (context, "dim-label");
+      gtk_widget_add_css_class (label, "dim-label");
       gtk_widget_set_halign (label, GTK_ALIGN_END);
       gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
       gtk_widget_set_margin_start (label, 10);
@@ -417,8 +398,10 @@ tab_grid_new ()
   GtkWidget *grid;
 
   grid = gtk_grid_new ();
-  gtk_widget_show (GTK_WIDGET (grid));
-  gtk_container_set_border_width (GTK_CONTAINER (grid), 20);
+  gtk_widget_set_margin_start (grid, 20);
+  gtk_widget_set_margin_end (grid, 20);
+  gtk_widget_set_margin_top (grid, 20);
+  gtk_widget_set_margin_bottom (grid, 20);
   gtk_grid_set_row_spacing (GTK_GRID (grid), 15);
 
   return grid;
@@ -437,12 +420,11 @@ tab_add (PpOptionsDialog *self,
 
   if (!grid_is_empty (grid))
     {
-      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-      gtk_widget_show (GTK_WIDGET (scrolled_window));
+      scrolled_window = gtk_scrolled_window_new ();
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                       GTK_POLICY_NEVER,
                                       GTK_POLICY_AUTOMATIC);
-      gtk_container_add (GTK_CONTAINER (scrolled_window), grid);
+      gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window), grid);
 
       id = gtk_notebook_append_page (self->notebook,
                                      scrolled_window,
@@ -516,7 +498,7 @@ populate_options_real (PpOptionsDialog *self)
 
   gtk_spinner_stop (self->spinner);
 
-  gtk_stack_set_visible_child (self->stack, GTK_WIDGET (self->main_box));
+  gtk_stack_set_visible_child (self->stack, self->main_box);
 
   if (self->ipp_attributes)
     {
@@ -880,9 +862,7 @@ pp_options_dialog_new (gchar   *printer_name,
 {
   PpOptionsDialog *self;
 
-  self = g_object_new (pp_options_dialog_get_type (),
-                       "use-header-bar", 1,
-                       NULL);
+  self = g_object_new (pp_options_dialog_get_type (), NULL);
 
   self->printer_name = g_strdup (printer_name);
 
@@ -944,6 +924,8 @@ pp_options_dialog_class_init (PpOptionsDialogClass *klass)
 
   gtk_widget_class_bind_template_callback (widget_class, category_selection_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, test_page_cb);
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 void
