@@ -25,13 +25,14 @@
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
+#include <adwaita.h>
 #include <polkit/polkit.h>
 
-#include "list-box-helper.h"
 #include "cc-region-panel.h"
 #include "cc-region-resources.h"
 #include "cc-language-chooser.h"
 #include "cc-format-chooser.h"
+#include "cc-list-row.h"
 
 #include "cc-common-language.h"
 
@@ -46,24 +47,22 @@
 
 #define DEFAULT_LOCALE "en_US.utf-8"
 
+typedef enum {
+        USER,
+        SYSTEM,
+} CcLocaleTarget;
+
 struct _CcRegionPanel {
         CcPanel          parent_instance;
 
-        GtkLabel        *formats_label;
-        GtkListBox      *formats_list;
-        GtkListBoxRow   *formats_row;
+        CcListRow       *formats_row;
+        AdwBanner       *banner;
         GtkSizeGroup    *input_size_group;
-        GtkLabel        *login_label;
-        GtkLabel        *language_label;
-        GtkListBox      *language_list;
+        CcListRow       *login_formats_row;
+        GtkWidget       *login_group;
+        CcListRow       *login_language_row;
         GtkListBoxRow   *language_row;
-        GtkFrame        *language_section_frame;
-        GtkToggleButton *login_language_button;
-        GtkButton       *restart_button;
-        GtkRevealer     *restart_revealer;
-        GtkBox          *session_or_login_box;
 
-        gboolean     login;
         gboolean     login_auto_apply;
         GPermission *permission;
         GDBusProxy  *localed;
@@ -81,45 +80,7 @@ struct _CcRegionPanel {
 
 CC_PANEL_REGISTER (CcRegionPanel, cc_region_panel)
 
-static void update_region (CcRegionPanel *self, const gchar *region);
-
-static void
-cc_region_panel_finalize (GObject *object)
-{
-        CcRegionPanel *self = CC_REGION_PANEL (object);
-        GtkWidget *chooser;
-
-        if (self->user_manager) {
-                g_signal_handlers_disconnect_by_data (self->user_manager, self);
-                self->user_manager = NULL;
-        }
-
-        if (self->user) {
-                g_signal_handlers_disconnect_by_data (self->user, self);
-                self->user = NULL;
-        }
-
-        g_clear_object (&self->permission);
-        g_clear_object (&self->localed);
-        g_clear_object (&self->session);
-        g_clear_object (&self->locale_settings);
-        g_free (self->language);
-        g_free (self->region);
-        g_free (self->system_language);
-        g_free (self->system_region);
-
-        chooser = g_object_get_data (G_OBJECT (self), "input-chooser");
-        if (chooser)
-                gtk_widget_destroy (chooser);
-
-        G_OBJECT_CLASS (cc_region_panel_parent_class)->finalize (object);
-}
-
-static const char *
-cc_region_panel_get_help_uri (CcPanel *panel)
-{
-        return "help:gnome-help/prefs-language";
-}
+/* Auxiliary methods */
 
 static GFile *
 get_needs_restart_file (void)
@@ -160,18 +121,16 @@ set_restart_notification_visible (CcRegionPanel *self,
 
         if (locale) {
                 new_locale = newlocale (LC_MESSAGES_MASK, locale, (locale_t) 0);
-                if (new_locale == (locale_t) 0)
-                        g_warning ("Failed to create locale %s: %s", locale, g_strerror (errno));
-                else
+                if (new_locale != (locale_t) 0) {
                         current_locale = uselocale (new_locale);
+                        uselocale (current_locale);
+                        freelocale (new_locale);
+                } else {
+                        g_warning ("Failed to create locale %s: %s", locale, g_strerror (errno));
+                }
         }
 
-        gtk_revealer_set_reveal_child (self->restart_revealer, visible);
-
-        if (locale && new_locale != (locale_t) 0) {
-                uselocale (current_locale);
-                freelocale (new_locale);
-        }
+        adw_banner_set_revealed (self->banner, visible);
 
         file = get_needs_restart_file ();
 
@@ -273,7 +232,38 @@ maybe_notify (CcRegionPanel *self,
                            mnd);
 }
 
-static void set_localed_locale (CcRegionPanel *self);
+static void
+set_localed_locale (CcRegionPanel *self)
+{
+        g_autoptr(GVariantBuilder) b = NULL;
+        g_autofree gchar *lang_value = NULL;
+        g_autofree gchar *lang = NULL;
+
+        cc_common_language_get_locale (self->system_language, &lang);
+
+        b = g_variant_builder_new (G_VARIANT_TYPE ("as"));
+        lang_value = g_strconcat ("LANG=", lang, NULL);
+        g_variant_builder_add (b, "s", lang_value);
+
+        g_free (lang_value);
+
+        lang_value = g_strconcat ("LANGUAGE=", self->system_language, NULL);
+        g_variant_builder_add (b, "s", lang_value);
+
+        const gchar *format_categories[] = { "LC_NUMERIC", "LC_TIME",
+           "LC_MONETARY", "LC_PAPER", "LC_IDENTIFICATION", "LC_NAME",
+           "LC_ADDRESS", "LC_TELEPHONE", "LC_MEASUREMENT", NULL };
+        for (int i = 0; format_categories[i] != NULL; i++) {
+                g_autofree gchar *s = NULL;
+                s = g_strconcat (format_categories[i], "=", self->system_region, NULL);
+                g_variant_builder_add (b, "s", s);
+        }
+        g_dbus_proxy_call (self->localed,
+                           "SetLocale",
+                           g_variant_new ("(asb)", b, TRUE),
+                           G_DBUS_CALL_FLAGS_NONE,
+                           -1, NULL, NULL, NULL);
+}
 
 static void
 set_system_language (CcRegionPanel *self,
@@ -289,39 +279,28 @@ set_system_language (CcRegionPanel *self,
 }
 
 static void
-update_language (CcRegionPanel *self,
-                 const gchar   *language)
+update_language (CcRegionPanel  *self,
+                 CcLocaleTarget  target,
+                 const gchar    *language)
 {
-        if (self->login) {
-                set_system_language (self, language);
-        } else {
+        g_debug ("Setting language to %s", language);
+
+        switch (target) {
+        case USER:
                 if (g_strcmp0 (language, self->language) == 0)
                         return;
                 act_user_set_language (self->user, language);
                 if (self->login_auto_apply)
                         set_system_language (self, language);
                 maybe_notify (self, LC_MESSAGES, language);
+                break;
+
+        case SYSTEM:
+                set_system_language (self, language);
+                break;
         }
 }
 
-static void
-language_response (CcRegionPanel     *self,
-                   gint               response_id,
-                   CcLanguageChooser *chooser)
-{
-        const gchar *language;
-
-        if (response_id == GTK_RESPONSE_OK) {
-                language = cc_language_chooser_get_language (chooser);
-                update_language (self, language);
-
-                /* Keep format strings consistent with the user's language */
-                update_region (self, NULL);
-        }
-
-        gtk_widget_destroy (GTK_WIDGET (chooser));
-}
-
 static void
 set_system_region (CcRegionPanel *self,
                    const gchar   *region)
@@ -336,24 +315,150 @@ set_system_region (CcRegionPanel *self,
 }
 
 static void
-update_region (CcRegionPanel *self,
-               const gchar   *region)
+set_formats_locale (const gchar *formats_locale)
 {
-        if (self->login) {
-                set_system_region (self, region);
-        } else {
+        g_autoptr(GDBusProxy) proxy = NULL;
+        g_autofree gchar *user_path = NULL;
+        g_autoptr(GVariant) ret = NULL;
+        g_autoptr(GError) error = NULL;
+
+        user_path = g_strdup_printf ("/org/freedesktop/Accounts/User%i", getuid ());
+        proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                               G_DBUS_PROXY_FLAGS_NONE,
+                                               NULL,
+                                               "org.freedesktop.Accounts",
+                                               user_path,
+                                               "org.freedesktop.Accounts.User",
+                                               NULL,
+                                               &error);
+        if (!proxy) {
+                g_warning ("Couldn't get accountsservice proxy for %s: %s", user_path, error->message);
+                return;
+        }
+
+        ret = g_dbus_proxy_call_sync (proxy,
+                                      "SetFormatsLocale",
+                                      g_variant_new ("(s)", formats_locale),
+                                      G_DBUS_CALL_FLAGS_NONE,
+                                      -1,
+                                      NULL,
+                                      &error);
+        if (!ret)
+                g_warning ("Couldn't set FormatsLocale: %s", error->message);
+}
+
+
+static void
+update_region (CcRegionPanel  *self,
+               CcLocaleTarget  target,
+               const gchar    *region)
+{
+        switch (target) {
+        case USER:
                 if (g_strcmp0 (region, self->region) == 0)
                         return;
                 if (region == NULL || region[0] == '\0')
                         g_settings_reset (self->locale_settings, KEY_REGION);
-                else
+                else {
+                        set_formats_locale (region);
                         g_settings_set_string (self->locale_settings, KEY_REGION, region);
+                }
                 if (self->login_auto_apply)
                         set_system_region (self, region);
+
+                if (region == NULL || region[0] == '\0') {
+                        // Formats (region) are being reset as part of changing the language,
+                        // and that already triggers the notification check.
+                        return;
+                }
+
                 maybe_notify (self, LC_TIME, region);
+                break;
+
+        case SYSTEM:
+                set_system_region (self, region);
+                break;
         }
 }
 
+static void
+language_response (CcRegionPanel     *self,
+                   gint               response_id,
+                   CcLanguageChooser *chooser)
+{
+        const gchar *language;
+
+        if (response_id == GTK_RESPONSE_OK) {
+                CcLocaleTarget target;
+
+                language = cc_language_chooser_get_language (chooser);
+                target = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (chooser), "target"));
+                update_language (self, target, language);
+
+                /* Keep format strings consistent with the user's language */
+                /* ... or don't. This idea does not apply in Ubuntu. */
+                // update_region (self, target, NULL);
+        }
+
+        gtk_window_destroy (GTK_WINDOW (chooser));
+}
+
+static const gchar *
+get_effective_language (CcRegionPanel  *self,
+                        CcLocaleTarget  target)
+{
+        switch (target) {
+        case USER:
+                return self->language;
+        case SYSTEM:
+                return self->system_language;
+        default:
+                g_assert_not_reached ();
+        }
+}
+
+static void
+show_language_chooser (CcRegionPanel  *self,
+                       CcLocaleTarget  target)
+{
+        CcLanguageChooser *chooser;
+        CcShell *shell;
+
+        shell = cc_panel_get_shell (CC_PANEL (self));
+        chooser = cc_language_chooser_new ();
+        gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (cc_shell_get_toplevel (shell)));
+        cc_language_chooser_set_language (chooser, get_effective_language (self, target));
+        g_object_set_data (G_OBJECT (chooser), "target", GINT_TO_POINTER (target));
+        g_signal_connect_object (chooser, "response",
+                                 G_CALLBACK (language_response), self, G_CONNECT_SWAPPED);
+        gtk_window_present (GTK_WINDOW (chooser));
+}
+
+static const gchar *
+get_effective_region (CcRegionPanel  *self,
+                      CcLocaleTarget  target)
+{
+        const gchar *region = NULL;
+
+        switch (target) {
+        case USER:
+                region = self->region;
+                break;
+
+        case SYSTEM:
+                region = self->system_region;
+                break;
+        }
+
+        /* Region setting might be empty - show the language because
+         * that's what LC_TIME and others will effectively be when the
+         * user logs in again. */
+        if (region == NULL || region[0] == '\0')
+                region = get_effective_language (self, target);
+
+        return region;
+}
+
 static void
 format_response (CcRegionPanel   *self,
                  gint             response_id,
@@ -362,79 +467,29 @@ format_response (CcRegionPanel   *self,
         const gchar *region;
 
         if (response_id == GTK_RESPONSE_OK) {
+                CcLocaleTarget target;
+
                 region = cc_format_chooser_get_region (chooser);
-                update_region (self, region);
+                target = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (chooser), "target"));
+
+                update_region (self, target, region);
         }
 
-        gtk_widget_destroy (GTK_WIDGET (chooser));
-}
-
-static const gchar *
-get_effective_language (CcRegionPanel *self)
-{
-        if (self->login)
-                return self->system_language;
-        else
-                return self->language;
-}
-
-static gboolean
-on_focus_out_event (GtkWindow *window,
-                    GdkEvent  *event)
-{
-  gtk_window_close(window);  
-
-  return TRUE;
+        gtk_window_destroy (GTK_WINDOW (chooser));
 }
 
 static void
-show_language_chooser (CcRegionPanel *self)
-{
-        CcLanguageChooser *chooser;
-
-        chooser = cc_language_chooser_new ();
-        gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
-        g_signal_connect (GTK_WINDOW (chooser),
-                    "focus-out-event",
-                    G_CALLBACK(on_focus_out_event),
-                    GTK_WINDOW (chooser));
-        cc_language_chooser_set_language (chooser, get_effective_language (self));
-        g_signal_connect_object (chooser, "response",
-                                 G_CALLBACK (language_response), self, G_CONNECT_SWAPPED);
-        gtk_window_present (GTK_WINDOW (chooser));
-}
-
-static const gchar *
-get_effective_region (CcRegionPanel *self)
-{
-        const gchar *region;
-
-        if (self->login)
-                region = self->system_region;
-        else
-                region = self->region;
-
-        /* Region setting might be empty - show the language because
-         * that's what LC_TIME and others will effectively be when the
-         * user logs in again. */
-        if (region == NULL || region[0] == '\0')
-                region = get_effective_language (self);
-
-        return region;
-}
-
-static void
-show_region_chooser (CcRegionPanel *self)
+show_region_chooser (CcRegionPanel  *self,
+                     CcLocaleTarget  target)
 {
         CcFormatChooser *chooser;
+        CcShell *shell;
 
+        shell = cc_panel_get_shell (CC_PANEL (self));
         chooser = cc_format_chooser_new ();
-        gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
-        g_signal_connect (GTK_WINDOW (chooser),
-                    "focus-out-event",
-                    G_CALLBACK(on_focus_out_event),
-                    GTK_WINDOW (chooser));
-        cc_format_chooser_set_region (chooser, get_effective_region (self));
+        gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (cc_shell_get_toplevel (shell)));
+        cc_format_chooser_set_region (chooser, get_effective_region (self, target));
+        g_object_set_data (G_OBJECT (chooser), "target", GINT_TO_POINTER (target));
         g_signal_connect_object (chooser, "response",
                                  G_CALLBACK (format_response), self, G_CONNECT_SWAPPED);
         gtk_window_present (GTK_WINDOW (chooser));
@@ -451,7 +506,7 @@ permission_acquired (GPermission *permission, GAsyncResult *res, const gchar *ac
                 return FALSE;
         }
 
-        return FALSE;
+        return TRUE;
 }
 
 static void
@@ -459,7 +514,7 @@ choose_language_permission_cb (GObject *source, GAsyncResult *res, gpointer user
 {
         CcRegionPanel *self = user_data;
         if (permission_acquired (G_PERMISSION (source), res, "choose language"))
-                show_language_chooser (self);
+                show_language_chooser (self, SYSTEM);
 }
 
 static void
@@ -467,38 +522,13 @@ choose_region_permission_cb (GObject *source, GAsyncResult *res, gpointer user_d
 {
         CcRegionPanel *self = user_data;
         if (permission_acquired (G_PERMISSION (source), res, "choose region"))
-                show_region_chooser (self);
+                show_region_chooser (self, SYSTEM);
 }
 
 static void
-activate_language_row (CcRegionPanel *self,
-                       GtkListBoxRow *row)
+update_user_region_row (CcRegionPanel *self)
 {
-        if (row == self->language_row) {
-                if (!self->login || g_permission_get_allowed (self->permission)) {
-                        show_language_chooser (self);
-                } else if (g_permission_get_can_acquire (self->permission)) {
-                        g_permission_acquire_async (self->permission,
-                                                    cc_panel_get_cancellable (CC_PANEL (self)),
-                                                    choose_language_permission_cb,
-                                                    self);
-                }
-        } else if (row == self->formats_row) {
-                if (!self->login || g_permission_get_allowed (self->permission)) {
-                        show_region_chooser (self);
-                } else if (g_permission_get_can_acquire (self->permission)) {
-                        g_permission_acquire_async (self->permission,
-                                                    cc_panel_get_cancellable (CC_PANEL (self)),
-                                                    choose_region_permission_cb,
-                                                    self);
-                }
-        }
-}
-
-static void
-update_region_label (CcRegionPanel *self)
-{
-        const gchar *region = get_effective_region (self);
+        const gchar *region = get_effective_region (self, USER);
         g_autofree gchar *name = NULL;
 
         if (region)
@@ -507,49 +537,52 @@ update_region_label (CcRegionPanel *self)
         if (!name)
                 name = gnome_get_country_from_locale (DEFAULT_LOCALE, DEFAULT_LOCALE);
 
-        gtk_label_set_label (self->formats_label, name);
+        adw_action_row_set_subtitle (ADW_ACTION_ROW (self->formats_row), name);
 }
 
 static void
-update_region_from_setting (CcRegionPanel *self)
+update_user_language_row (CcRegionPanel *self)
 {
-        g_free (self->region);
-        self->region = g_settings_get_string (self->locale_settings, KEY_REGION);
-        update_region_label (self);
-}
-
-static void
-update_language_label (CcRegionPanel *self)
-{
-        const gchar *language = get_effective_language (self);
         g_autofree gchar *name = NULL;
 
-        if (language)
-                name = gnome_get_language_from_locale (language, language);
+        if (self->language)
+                name = gnome_get_language_from_locale (self->language, self->language);
 
         if (!name)
                 name = gnome_get_language_from_locale (DEFAULT_LOCALE, DEFAULT_LOCALE);
 
-        gtk_label_set_label (self->language_label, name);
+        adw_action_row_set_subtitle (ADW_ACTION_ROW (self->language_row), name);
 
         /* Formats will change too if not explicitly set. */
-        update_region_label (self);
+        update_user_region_row (self);
 }
 
 static void
 update_language_from_user (CcRegionPanel *self)
 {
         const gchar *language = NULL;
+        g_autofree gchar *locale = NULL;
 
-        if (act_user_is_loaded (self->user))
+        if (act_user_is_loaded (self->user)) {
                 language = act_user_get_language (self->user);
+                cc_common_language_get_locale (language, &locale);
+        }
 
-        if (language == NULL || *language == '\0')
-                language = setlocale (LC_MESSAGES, NULL);
+        if (language == NULL || *language == '\0') {
+                locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+        }
 
         g_free (self->language);
-        self->language = g_strdup (language);
-        update_language_label (self);
+        self->language = g_steal_pointer (&locale);
+        update_user_language_row (self);
+}
+
+static void
+update_region_from_setting (CcRegionPanel *self)
+{
+        g_free (self->region);
+        self->region = cc_common_language_get_property ("FormatsLocale");
+        update_user_region_row (self);
 }
 
 static void
@@ -565,26 +598,96 @@ setup_language_section (CcRegionPanel *self)
         g_signal_connect_object (self->locale_settings, "changed::" KEY_REGION,
                                  G_CALLBACK (update_region_from_setting), self, G_CONNECT_SWAPPED);
 
-        gtk_list_box_set_header_func (self->language_list,
-                                      cc_list_box_update_header_func,
-                                      NULL, NULL);
-        g_signal_connect_object (self->language_list, "row-activated",
-                                 G_CALLBACK (activate_language_row), self, G_CONNECT_SWAPPED);
-
-        gtk_list_box_set_header_func (self->formats_list,
-                                      cc_list_box_update_header_func,
-                                      NULL, NULL);
-        g_signal_connect_object (self->formats_list, "row-activated",
-                                 G_CALLBACK (activate_language_row), self, G_CONNECT_SWAPPED);
-
         update_language_from_user (self);
         update_region_from_setting (self);
 }
 
+
 static void
-on_localed_properties_changed (CcRegionPanel  *self,
+show_language_support (CcRegionPanel *self)
+{
+        g_autoptr(GAppInfo) app_info = NULL;
+        g_autoptr(GdkAppLaunchContext) ctx = NULL;
+        g_autoptr(GError) error = NULL;
+
+        app_info = G_APP_INFO (g_desktop_app_info_new ("gnome-language-selector.desktop"));
+
+        if (app_info == NULL) {
+                g_warning ("Failed to launch language-selector: couldn't create GDesktopAppInfo");
+                return;
+        }
+
+        ctx = gdk_display_get_app_launch_context (gdk_display_get_default ());
+        if (!g_app_info_launch (app_info, NULL, G_APP_LAUNCH_CONTEXT (ctx), &error))
+                g_warning ("Failed to launch language-selector: %s", error->message);
+}
+
+static void
+update_login_region (CcRegionPanel *self)
+{
+        g_autofree gchar *name = NULL;
+
+        if (self->system_region)
+                name = gnome_get_country_from_locale (self->system_region, self->system_region);
+        else if (self->system_language)
+                name = gnome_get_country_from_locale (self->system_language, self->system_language);
+
+        if (!name)
+                name = gnome_get_country_from_locale (DEFAULT_LOCALE, DEFAULT_LOCALE);
+
+        adw_action_row_set_subtitle (ADW_ACTION_ROW (self->login_formats_row), name);
+}
+
+static void
+update_login_language (CcRegionPanel *self)
+{
+        g_autofree gchar *name = NULL;
+
+        if (self->system_language)
+                name = gnome_get_language_from_locale (self->system_language, self->system_language);
+
+        if (!name)
+                name = gnome_get_language_from_locale (DEFAULT_LOCALE, DEFAULT_LOCALE);
+
+        adw_action_row_set_subtitle (ADW_ACTION_ROW (self->login_language_row), name);
+        update_login_region (self);
+}
+
+static void
+set_login_button_visibility (CcRegionPanel *self)
+{
+        gboolean has_multiple_users;
+        gboolean loaded;
+
+        g_object_get (self->user_manager, "is-loaded", &loaded, NULL);
+        if (!loaded)
+          return;
+
+        g_object_get (self->user_manager, "has-multiple-users", &has_multiple_users, NULL);
+
+        self->login_auto_apply = !has_multiple_users && g_permission_get_allowed (self->permission);
+        gtk_widget_set_visible (self->login_group, !self->login_auto_apply);
+
+        g_signal_handlers_disconnect_by_func (self->user_manager, set_login_button_visibility, self);
+}
+
+/* Callbacks */
+
+static gchar *
+strip_quotes (const gchar *str)
+{
+        if ((g_str_has_prefix (str, "\"") && g_str_has_suffix (str, "\""))
+          || (g_str_has_prefix (str, "'") && g_str_has_suffix (str, "'")))
+                return g_strndup (str + 1, strlen (str) - 2);
+        else
+                return g_strdup (str);
+}
+
+static void
+on_localed_properties_changed (GDBusProxy     *localed_proxy,
                                GVariant       *changed_properties,
-                               const gchar   **invalidated_properties)
+                               const gchar   **invalidated_properties,
+                               CcRegionPanel  *self)
 {
         g_autoptr(GVariant) v = NULL;
 
@@ -593,69 +696,45 @@ on_localed_properties_changed (CcRegionPanel  *self,
                 g_autofree const gchar **strv = NULL;
                 gsize len;
                 gint i;
-                const gchar *lang, *messages, *time;
+                g_autofree gchar *lang;
+                g_autofree gchar *language;
+                g_autofree gchar *messages;
+                g_autofree gchar *time;
 
                 strv = g_variant_get_strv (v, &len);
 
-                lang = messages = time = NULL;
+                lang = language = messages = time = NULL;
                 for (i = 0; strv[i]; i++) {
                         if (g_str_has_prefix (strv[i], "LANG=")) {
-                                lang = strv[i] + strlen ("LANG=");
+                                lang = strip_quotes (strv[i] + strlen ("LANG="));
+                        } else if (g_str_has_prefix (strv[i], "LANGUAGE=")) {
+                                g_autofree gchar *tmp = strip_quotes (strv[i] + strlen ("LANGUAGE="));
+                                g_auto(GStrv) tokens = g_strsplit (tmp, ":", 2);
+                                language = g_strdup (tokens[0]);
                         } else if (g_str_has_prefix (strv[i], "LC_MESSAGES=")) {
-                                messages = strv[i] + strlen ("LC_MESSAGES=");
+                                messages = strip_quotes (strv[i] + strlen ("LC_MESSAGES="));
                         } else if (g_str_has_prefix (strv[i], "LC_TIME=")) {
-                                time = strv[i] + strlen ("LC_TIME=");
+                                time = strip_quotes (strv[i] + strlen ("LC_TIME="));
                         }
                 }
                 if (!lang) {
-                        lang = setlocale (LC_MESSAGES, NULL);
+                        lang = g_strdup ("en_US.UTF-8");
                 }
-                if (!messages) {
-                        messages = lang;
+                if (!language) {
+                        if (messages)
+                                language = g_strdup (messages);
+                        else
+                                language = g_strdup (lang);
                 }
                 g_free (self->system_language);
-                self->system_language = g_strdup (messages);
+                self->system_language = g_steal_pointer (&language);
                 g_free (self->system_region);
                 self->system_region = g_strdup (time);
 
-                update_language_label (self);
+                update_login_language (self);
         }
 }
 
-static void
-set_localed_locale (CcRegionPanel *self)
-{
-        g_autoptr(GVariantBuilder) b = NULL;
-        g_autofree gchar *lang_value = NULL;
-
-        b = g_variant_builder_new (G_VARIANT_TYPE ("as"));
-        lang_value = g_strconcat ("LANG=", self->system_language, NULL);
-        g_variant_builder_add (b, "s", lang_value);
-
-        if (self->system_region != NULL) {
-                g_autofree gchar *time_value = NULL;
-                g_autofree gchar *numeric_value = NULL;
-                g_autofree gchar *monetary_value = NULL;
-                g_autofree gchar *measurement_value = NULL;
-                g_autofree gchar *paper_value = NULL;
-                time_value = g_strconcat ("LC_TIME=", self->system_region, NULL);
-                g_variant_builder_add (b, "s", time_value);
-                numeric_value = g_strconcat ("LC_NUMERIC=", self->system_region, NULL);
-                g_variant_builder_add (b, "s", numeric_value);
-                monetary_value = g_strconcat ("LC_MONETARY=", self->system_region, NULL);
-                g_variant_builder_add (b, "s", monetary_value);
-                measurement_value = g_strconcat ("LC_MEASUREMENT=", self->system_region, NULL);
-                g_variant_builder_add (b, "s", measurement_value);
-                paper_value = g_strconcat ("LC_PAPER=", self->system_region, NULL);
-                g_variant_builder_add (b, "s", paper_value);
-        }
-        g_dbus_proxy_call (self->localed,
-                           "SetLocale",
-                           g_variant_new ("(asb)", b, TRUE),
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1, NULL, NULL, NULL);
-}
-
 static void
 localed_proxy_ready (GObject      *source,
                      GAsyncResult *res,
@@ -675,52 +754,22 @@ localed_proxy_ready (GObject      *source,
 
         self->localed = proxy;
 
-        g_signal_connect_object (self->localed, "g-properties-changed",
-                                 G_CALLBACK (on_localed_properties_changed), self, G_CONNECT_SWAPPED);
-        on_localed_properties_changed (self, NULL, NULL);
+        g_signal_connect_object (self->localed,
+                                 "g-properties-changed",
+                                 G_CALLBACK (on_localed_properties_changed),
+                                 self,
+                                 0);
+
+        on_localed_properties_changed (self->localed, NULL, NULL, self);
 }
 
 static void
-login_changed (CcRegionPanel *self)
-{
-        gboolean can_acquire;
-
-        self->login = gtk_toggle_button_get_active (self->login_language_button);
-        gtk_widget_set_visible (GTK_WIDGET (self->login_label), self->login);
-
-        can_acquire = self->permission &&
-                (g_permission_get_allowed (self->permission) ||
-                 g_permission_get_can_acquire (self->permission));
-        /* FIXME: insensitive doesn't look quite right for this */
-        gtk_widget_set_sensitive (GTK_WIDGET (self->language_section_frame), !self->login || can_acquire);
-
-        update_language_label (self);
-}
-
-static void
-set_login_button_visibility (CcRegionPanel *self)
-{
-        gboolean has_multiple_users;
-        gboolean loaded;
-
-        g_object_get (self->user_manager, "is-loaded", &loaded, NULL);
-        if (!loaded)
-          return;
-
-        g_object_get (self->user_manager, "has-multiple-users", &has_multiple_users, NULL);
-
-        self->login_auto_apply = !has_multiple_users && g_permission_get_allowed (self->permission);
-        gtk_widget_set_visible (GTK_WIDGET (self->session_or_login_box), !self->login_auto_apply);
-
-        g_signal_handlers_disconnect_by_func (self->user_manager, set_login_button_visibility, self);
-}
-
-static void
-setup_login_button (CcRegionPanel *self)
+setup_login_permission (CcRegionPanel *self)
 {
         g_autoptr(GDBusConnection) bus = NULL;
-        gboolean loaded;
         g_autoptr(GError) error = NULL;
+        gboolean can_acquire;
+        gboolean loaded;
 
         self->permission = polkit_permission_new_sync ("org.freedesktop.locale1.set-locale", NULL, NULL, &error);
         if (self->permission == NULL) {
@@ -740,15 +789,17 @@ setup_login_button (CcRegionPanel *self)
                           (GAsyncReadyCallback) localed_proxy_ready,
                           self);
 
-        g_signal_connect_object (self->login_language_button, "notify::active",
-                                 G_CALLBACK (login_changed), self, G_CONNECT_SWAPPED);
-
         g_object_get (self->user_manager, "is-loaded", &loaded, NULL);
         if (loaded)
                 set_login_button_visibility (self);
         else
                 g_signal_connect_object (self->user_manager, "notify::is-loaded",
                                          G_CALLBACK (set_login_button_visibility), self, G_CONNECT_SWAPPED);
+
+        can_acquire = self->permission &&
+                (g_permission_get_allowed (self->permission) ||
+                 g_permission_get_can_acquire (self->permission));
+        gtk_widget_set_sensitive (self->login_group, can_acquire);
 }
 
 static void
@@ -771,6 +822,86 @@ session_proxy_ready (GObject      *source,
         self->session = proxy;
 }
 
+static void
+on_login_formats_row_activated_cb (CcRegionPanel *self)
+{
+        if (g_permission_get_allowed (self->permission)) {
+                show_region_chooser (self, SYSTEM);
+        } else if (g_permission_get_can_acquire (self->permission)) {
+                g_permission_acquire_async (self->permission,
+                                            cc_panel_get_cancellable (CC_PANEL (self)),
+                                            choose_region_permission_cb,
+                                            self);
+        }
+}
+
+static void
+on_login_language_row_activated_cb (CcRegionPanel *self)
+{
+        if (g_permission_get_allowed (self->permission)) {
+                show_language_chooser (self, SYSTEM);
+        } else if (g_permission_get_can_acquire (self->permission)) {
+                g_permission_acquire_async (self->permission,
+                                            cc_panel_get_cancellable (CC_PANEL (self)),
+                                            choose_language_permission_cb,
+                                            self);
+        }
+}
+
+static void
+on_user_formats_row_activated_cb (CcRegionPanel *self)
+{
+        show_region_chooser (self, USER);
+}
+
+static void
+on_user_language_row_activated_cb (CcRegionPanel *self)
+{
+        show_language_chooser (self, USER);
+}
+
+/* CcPanel overrides */
+
+static const char *
+cc_region_panel_get_help_uri (CcPanel *panel)
+{
+        return "help:gnome-help/prefs-language";
+}
+
+/* GObject overrides */
+
+static void
+cc_region_panel_finalize (GObject *object)
+{
+        CcRegionPanel *self = CC_REGION_PANEL (object);
+        GtkWidget *chooser;
+
+        if (self->user_manager) {
+                g_signal_handlers_disconnect_by_data (self->user_manager, self);
+                self->user_manager = NULL;
+        }
+
+        if (self->user) {
+                g_signal_handlers_disconnect_by_data (self->user, self);
+                self->user = NULL;
+        }
+
+        g_clear_object (&self->permission);
+        g_clear_object (&self->localed);
+        g_clear_object (&self->session);
+        g_clear_object (&self->locale_settings);
+        g_free (self->language);
+        g_free (self->region);
+        g_free (self->system_language);
+        g_free (self->system_region);
+
+        chooser = g_object_get_data (G_OBJECT (self), "input-chooser");
+        if (chooser)
+                gtk_window_destroy (GTK_WINDOW (chooser));
+
+        G_OBJECT_CLASS (cc_region_panel_parent_class)->finalize (object);
+}
+
 static void
 cc_region_panel_class_init (CcRegionPanelClass * klass)
 {
@@ -782,22 +913,23 @@ cc_region_panel_class_init (CcRegionPanelClass * klass)
 
         object_class->finalize = cc_region_panel_finalize;
 
+        g_type_ensure (CC_TYPE_LIST_ROW);
+
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/region/cc-region-panel.ui");
 
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, formats_label);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, formats_list);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, formats_row);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_label);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_label);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_list);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, banner);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_formats_row);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_group);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_language_row);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_row);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_section_frame);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_language_button);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, restart_button);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, restart_revealer);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, session_or_login_box);
 
+        gtk_widget_class_bind_template_callback (widget_class, on_login_formats_row_activated_cb);
+        gtk_widget_class_bind_template_callback (widget_class, on_login_language_row_activated_cb);
+        gtk_widget_class_bind_template_callback (widget_class, on_user_formats_row_activated_cb);
+        gtk_widget_class_bind_template_callback (widget_class, on_user_language_row_activated_cb);
         gtk_widget_class_bind_template_callback (widget_class, restart_now);
+        gtk_widget_class_bind_template_callback (widget_class, show_language_support);
 }
 
 static void
@@ -821,7 +953,7 @@ cc_region_panel_init (CcRegionPanel *self)
                                   session_proxy_ready,
                                   self);
 
-        setup_login_button (self);
+        setup_login_permission (self);
         setup_language_section (self);
 
         needs_restart_file = get_needs_restart_file ();
