@@ -1,5 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
- *
+/*
  * Copyright (C) 2008 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
@@ -30,44 +29,44 @@
 #include <pulse/pulseaudio.h>
 #include <gvc-mixer-control.h>
 
-#include "list-box-helper.h"
-#include "cc-alert-chooser.h"
+#include "cc-alert-chooser-window.h"
 #include "cc-balance-slider.h"
 #include "cc-device-combo-box.h"
 #include "cc-fade-slider.h"
 #include "cc-level-bar.h"
-#include "cc-output-test-dialog.h"
+#include "cc-output-test-window.h"
 #include "cc-profile-combo-box.h"
 #include "cc-sound-panel.h"
 #include "cc-sound-resources.h"
-#include "cc-stream-list-box.h"
 #include "cc-subwoofer-slider.h"
+#include "cc-volume-levels-window.h"
 #include "cc-volume-slider.h"
 
 struct _CcSoundPanel
 {
   CcPanel            parent_instance;
 
-  CcBalanceSlider   *balance_slider;
-  GtkListBoxRow     *fade_row;
-  CcFadeSlider      *fade_slider;
-  CcDeviceComboBox  *input_device_combo_box;
-  CcLevelBar        *input_level_bar;
-  GtkListBox        *input_list_box;
-  CcProfileComboBox *input_profile_combo_box;
-  GtkListBoxRow     *input_profile_row;
-  CcVolumeSlider    *input_volume_slider;
-  GtkSizeGroup      *label_size_group;
-  CcDeviceComboBox  *output_device_combo_box;
-  GtkListStore      *output_device_model;
-  CcLevelBar        *output_level_bar;
-  GtkListBox        *output_list_box;
-  CcProfileComboBox *output_profile_combo_box;
-  GtkListBoxRow     *output_profile_row;
-  CcVolumeSlider    *output_volume_slider;
-  CcStreamListBox   *stream_list_box;
-  GtkListBoxRow     *subwoofer_row;
-  CcSubwooferSlider *subwoofer_slider;
+  AdwPreferencesGroup *output_group;
+  CcLevelBar          *output_level_bar;
+  CcDeviceComboBox    *output_device_combo_box;
+  AdwPreferencesRow   *output_profile_row;
+  CcProfileComboBox   *output_profile_combo_box;
+  CcVolumeSlider      *output_volume_slider;
+  CcBalanceSlider     *balance_slider;
+  AdwSwitchRow        *allow_amplify_row;
+  AdwPreferencesRow   *fade_row;
+  CcFadeSlider        *fade_slider;
+  AdwPreferencesRow   *subwoofer_row;
+  CcSubwooferSlider   *subwoofer_slider;
+  AdwPreferencesGroup *output_no_devices_group;
+  AdwPreferencesGroup *input_group;
+  CcLevelBar          *input_level_bar;
+  CcDeviceComboBox    *input_device_combo_box;
+  AdwPreferencesRow   *input_profile_row;
+  CcProfileComboBox   *input_profile_combo_box;
+  CcVolumeSlider      *input_volume_slider;
+  AdwPreferencesGroup *input_no_devices_group;
+  GtkLabel            *alert_sound_label;
 
   GvcMixerControl   *mixer_control;
   GSettings         *sound_settings;
@@ -83,6 +82,13 @@ enum
 
 #define KEY_SOUNDS_SCHEMA "org.gnome.desktop.sound"
 
+static void
+update_alert_sound_label (CcSoundPanel *self)
+{
+  const gchar *alert_name = get_selected_alert_display_name ();
+  gtk_label_set_label (self->alert_sound_label, alert_name);
+}
+
 static void
 allow_amplified_changed_cb (CcSoundPanel *self)
 {
@@ -98,7 +104,7 @@ set_output_stream (CcSoundPanel   *self,
   gboolean can_fade = FALSE, has_lfe = FALSE;
 
   cc_volume_slider_set_stream (self->output_volume_slider, stream, CC_STREAM_TYPE_OUTPUT);
-  cc_level_bar_set_stream (self->output_level_bar, stream, CC_STREAM_TYPE_OUTPUT);
+  cc_level_bar_set_stream (self->output_level_bar, stream);
 
   if (stream != NULL)
     {
@@ -122,6 +128,9 @@ output_device_changed_cb (CcSoundPanel *self)
 
   device = cc_device_combo_box_get_device (self->output_device_combo_box);
 
+  gtk_widget_set_visible (GTK_WIDGET (self->output_group), device != NULL);
+  gtk_widget_set_visible (GTK_WIDGET (self->output_no_devices_group), device == NULL);
+
   if (device != NULL)
     stream = gvc_mixer_control_get_stream_from_device (self->mixer_control, device);
 
@@ -136,7 +145,7 @@ set_input_stream (CcSoundPanel   *self,
                   GvcMixerStream *stream)
 {
   cc_volume_slider_set_stream (self->input_volume_slider, stream, CC_STREAM_TYPE_INPUT);
-  cc_level_bar_set_stream (self->input_level_bar, stream, CC_STREAM_TYPE_INPUT);
+  cc_level_bar_set_stream (self->input_level_bar, stream);
 }
 
 static void
@@ -147,6 +156,9 @@ input_device_changed_cb (CcSoundPanel *self)
 
   device = cc_device_combo_box_get_device (self->input_device_combo_box);
 
+  gtk_widget_set_visible (GTK_WIDGET (self->input_group), device != NULL);
+  gtk_widget_set_visible (GTK_WIDGET (self->input_no_devices_group), device == NULL);
+
   if (device != NULL)
     stream = gvc_mixer_control_get_stream_from_device (self->mixer_control, device);
 
@@ -196,34 +208,60 @@ input_device_update_cb (CcSoundPanel *self,
     set_input_stream (self, stream);
 }
 
-static gboolean
-on_focus_out_event (GtkWindow *window,
-                    GdkEvent  *event)
-{
-  gtk_window_close(window);  
-
-  return TRUE;
-}
-
 static void
 test_output_configuration_button_clicked_cb (CcSoundPanel *self)
 {
   GvcMixerUIDevice *device;
   GvcMixerStream *stream = NULL;
-  CcOutputTestDialog *dialog;
+  CcOutputTestWindow *window;
+  GtkWidget *toplevel;
+  CcShell *shell;
 
   device = cc_device_combo_box_get_device (self->output_device_combo_box);
   if (device != NULL)
     stream = gvc_mixer_control_get_stream_from_device (self->mixer_control, device);
 
-  dialog = cc_output_test_dialog_new (device, stream);
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
-  g_signal_connect (GTK_WINDOW (dialog),
-                    "focus-out-event",
-                    G_CALLBACK(on_focus_out_event),
-                    GTK_WINDOW (dialog));
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  shell = cc_panel_get_shell (CC_PANEL (self));
+  toplevel = cc_shell_get_toplevel (shell);
+
+  window = cc_output_test_window_new (stream);
+  gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (toplevel));
+  gtk_window_present (GTK_WINDOW (window));
+}
+
+static void
+volume_levels_activated_cb (CcSoundPanel *self)
+{
+  CcVolumeLevelsWindow *volume_levels;
+  GtkWindow *toplevel;
+  CcShell *shell;
+
+  shell = cc_panel_get_shell (CC_PANEL (self));
+  toplevel = GTK_WINDOW (cc_shell_get_toplevel (shell));
+
+  volume_levels = cc_volume_levels_window_new (self->mixer_control);
+  gtk_window_set_transient_for (GTK_WINDOW (volume_levels), toplevel);
+  gtk_window_present (GTK_WINDOW (volume_levels));
+}
+
+static void
+alert_sound_activated_cb (CcSoundPanel *self)
+{
+  CcAlertChooserWindow *alert_chooser;
+  GtkWindow *toplevel;
+  CcShell *shell;
+
+  shell = cc_panel_get_shell (CC_PANEL (self));
+  toplevel = GTK_WINDOW (cc_shell_get_toplevel (shell));
+
+  alert_chooser = cc_alert_chooser_window_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (alert_chooser), toplevel);
+
+  g_signal_connect_object (alert_chooser, "destroy",
+                           G_CALLBACK (update_alert_sound_label),
+                           self, G_CONNECT_SWAPPED);
+
+  gtk_window_present (GTK_WINDOW (alert_chooser));
 }
 
 static const char *
@@ -235,10 +273,10 @@ cc_sound_panel_get_help_uri (CcPanel *panel)
 static void
 cc_sound_panel_finalize (GObject *object)
 {
-  CcSoundPanel *panel = CC_SOUND_PANEL (object);
+  CcSoundPanel *self = CC_SOUND_PANEL (object);
 
-  g_clear_object (&panel->mixer_control);
-  g_clear_object (&panel->sound_settings);
+  g_clear_object (&self->mixer_control);
+  g_clear_object (&self->sound_settings);
 
   G_OBJECT_CLASS (cc_sound_panel_parent_class)->finalize (object);
 }
@@ -256,37 +294,39 @@ cc_sound_panel_class_init (CcSoundPanelClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/sound/cc-sound-panel.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_group);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_level_bar);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_device_combo_box);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_profile_row);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_profile_combo_box);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_volume_slider);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, allow_amplify_row);
   gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, balance_slider);
   gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, fade_row);
   gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, fade_slider);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_device_combo_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_level_bar);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_list_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_profile_combo_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_profile_row);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_volume_slider);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, label_size_group);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_device_combo_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_level_bar);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_list_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_profile_combo_box);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_profile_row);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_volume_slider);
-  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, stream_list_box);
   gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, subwoofer_row);
   gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, subwoofer_slider);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, output_no_devices_group);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_group);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_level_bar);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_device_combo_box);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_profile_row);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_profile_combo_box);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_volume_slider);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, input_no_devices_group);
+  gtk_widget_class_bind_template_child (widget_class, CcSoundPanel, alert_sound_label);
 
   gtk_widget_class_bind_template_callback (widget_class, input_device_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, output_device_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, test_output_configuration_button_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, volume_levels_activated_cb);
+  gtk_widget_class_bind_template_callback (widget_class, alert_sound_activated_cb);
 
-  g_type_ensure (CC_TYPE_ALERT_CHOOSER);
   g_type_ensure (CC_TYPE_BALANCE_SLIDER);
   g_type_ensure (CC_TYPE_DEVICE_COMBO_BOX);
   g_type_ensure (CC_TYPE_FADE_SLIDER);
   g_type_ensure (CC_TYPE_LEVEL_BAR);
   g_type_ensure (CC_TYPE_PROFILE_COMBO_BOX);
-  g_type_ensure (CC_TYPE_STREAM_LIST_BOX);
   g_type_ensure (CC_TYPE_SUBWOOFER_SLIDER);
   g_type_ensure (CC_TYPE_VOLUME_SLIDER);
 }
@@ -306,10 +346,20 @@ cc_sound_panel_init (CcSoundPanel *self)
                            G_CONNECT_SWAPPED);
   allow_amplified_changed_cb (self);
 
+  const gchar *desktop_list = g_getenv ("XDG_CURRENT_DESKTOP");
+  g_auto(GStrv) desktops = NULL;
+  if (desktop_list)
+    desktops = g_strsplit (desktop_list, ":", 0);
+  if (desktops && g_strv_contains ((const gchar * const *) desktops, "ubuntu")) {
+    g_settings_bind (self->sound_settings, "allow-volume-above-100-percent",
+                     self->allow_amplify_row, "active", G_SETTINGS_BIND_DEFAULT);
+  } else {
+    gtk_widget_set_visible (GTK_WIDGET (self->allow_amplify_row), FALSE);
+  }
+
   self->mixer_control = gvc_mixer_control_new ("GNOME Settings");
   gvc_mixer_control_open (self->mixer_control);
 
-  cc_stream_list_box_set_mixer_control (self->stream_list_box, self->mixer_control);
   cc_volume_slider_set_mixer_control (self->input_volume_slider, self->mixer_control);
   cc_volume_slider_set_mixer_control (self->output_volume_slider, self->mixer_control);
   cc_subwoofer_slider_set_mixer_control (self->subwoofer_slider, self->mixer_control);
@@ -325,4 +375,6 @@ cc_sound_panel_init (CcSoundPanel *self)
                            G_CALLBACK (input_device_update_cb),
                            self,
                            G_CONNECT_SWAPPED);
+
+  update_alert_sound_label (self);
 }
