@@ -43,47 +43,42 @@ extern GType cc_display_panel_get_type (void);
 extern GType cc_info_overview_panel_get_type (void);
 extern GType cc_keyboard_panel_get_type (void);
 extern GType cc_mouse_panel_get_type (void);
+extern GType cc_multitasking_panel_get_type (void);
 #ifdef BUILD_NETWORK
 extern GType cc_network_panel_get_type (void);
 extern GType cc_wifi_panel_get_type (void);
 #endif /* BUILD_NETWORK */
 extern GType cc_notifications_panel_get_type (void);
-extern GType cc_goa_panel_get_type (void);
+extern GType cc_online_accounts_panel_get_type (void);
 extern GType cc_power_panel_get_type (void);
 extern GType cc_printers_panel_get_type (void);
+extern GType cc_privacy_panel_get_type (void);
 extern GType cc_region_panel_get_type (void);
 extern GType cc_removable_media_panel_get_type (void);
 extern GType cc_search_panel_get_type (void);
 extern GType cc_sharing_panel_get_type (void);
 extern GType cc_sound_panel_get_type (void);
-#ifdef BUILD_THUNDERBOLT
-extern GType cc_bolt_panel_get_type (void);
-#endif /* BUILD_THUNDERBOLT */
 extern GType cc_ua_panel_get_type (void);
+extern GType cc_ubuntu_panel_get_type(void);
 extern GType cc_user_panel_get_type (void);
 #ifdef BUILD_WACOM
 extern GType cc_wacom_panel_get_type (void);
 #endif /* BUILD_WACOM */
-
-
-
-extern GType cc_location_panel_get_type (void);
-extern GType cc_camera_panel_get_type (void);
-extern GType cc_microphone_panel_get_type (void);
-extern GType cc_usage_panel_get_type (void);
-extern GType cc_lock_panel_get_type (void);
-extern GType cc_diagnostics_panel_get_type (void);
+#ifdef BUILD_WWAN
+extern GType cc_wwan_panel_get_type (void);
+#endif /* BUILD_WWAN */
+extern GType cc_ubuntu_panel_static_init_func (void);
 
 /* Static init functions */
-extern void cc_diagnostics_panel_static_init_func (void);
 #ifdef BUILD_NETWORK
 extern void cc_wifi_panel_static_init_func (void);
 #endif /* BUILD_NETWORK */
 #ifdef BUILD_WACOM
 extern void cc_wacom_panel_static_init_func (void);
 #endif /* BUILD_WACOM */
-
-
+#ifdef BUILD_WWAN
+extern void cc_wwan_panel_static_init_func (void);
+#endif /* BUILD_WWAN */
 
 #define PANEL_TYPE(name, get_type, init_func) { name, get_type, init_func }
 
@@ -95,49 +90,42 @@ extern void cc_wacom_panel_static_init_func (void);
 
 static CcPanelLoaderVtable default_panels[] =
 {
-  PANEL_TYPE("applications",     cc_applications_panel_get_type,         NULL),
-
+//  PANEL_TYPE("applications",     cc_applications_panel_get_type,         NULL),
+  PANEL_TYPE("background",       cc_background_panel_get_type,           NULL),
 #ifdef BUILD_BLUETOOTH
   PANEL_TYPE("bluetooth",        cc_bluetooth_panel_get_type,            NULL),
 #endif
-  
   PANEL_TYPE("color",            cc_color_panel_get_type,                NULL),
   PANEL_TYPE("datetime",         cc_date_time_panel_get_type,            NULL),
   PANEL_TYPE("default-apps",     cc_default_apps_panel_get_type,         NULL),
-  PANEL_TYPE("diagnostics",      cc_diagnostics_panel_get_type,          cc_diagnostics_panel_static_init_func),
   PANEL_TYPE("display",          cc_display_panel_get_type,              NULL),
   PANEL_TYPE("info-overview",    cc_info_overview_panel_get_type,        NULL),
   PANEL_TYPE("keyboard",         cc_keyboard_panel_get_type,             NULL),
-  PANEL_TYPE("location",         cc_location_panel_get_type,             NULL),
-
-  
   PANEL_TYPE("mouse",            cc_mouse_panel_get_type,                NULL),
-
+//  PANEL_TYPE("multitasking",     cc_multitasking_panel_get_type,         NULL),
 #ifdef BUILD_NETWORK
   PANEL_TYPE("network",          cc_network_panel_get_type,              NULL),
   PANEL_TYPE("wifi",             cc_wifi_panel_get_type,                 cc_wifi_panel_static_init_func),
 #endif
-  
-  PANEL_TYPE("online-accounts",  cc_goa_panel_get_type,                  NULL),
+//  PANEL_TYPE("notifications",    cc_notifications_panel_get_type,        NULL),
+  PANEL_TYPE("online-accounts",  cc_online_accounts_panel_get_type,      NULL),
   PANEL_TYPE("power",            cc_power_panel_get_type,                NULL),
   PANEL_TYPE("printers",         cc_printers_panel_get_type,             NULL),
+  PANEL_TYPE("privacy",          cc_privacy_panel_get_type,              NULL),
   PANEL_TYPE("region",           cc_region_panel_get_type,               NULL),
   PANEL_TYPE("removable-media",  cc_removable_media_panel_get_type,      NULL),
-
+//  PANEL_TYPE("search",           cc_search_panel_get_type,               NULL),
   PANEL_TYPE("sharing",          cc_sharing_panel_get_type,              NULL),
   PANEL_TYPE("sound",            cc_sound_panel_get_type,                NULL),
-#ifdef BUILD_THUNDERBOLT
-  PANEL_TYPE("thunderbolt",      cc_bolt_panel_get_type,                 NULL),
-#endif
+  PANEL_TYPE("ubuntu",           cc_ubuntu_panel_get_type,               cc_ubuntu_panel_static_init_func),
   PANEL_TYPE("universal-access", cc_ua_panel_get_type,                   NULL),
-  PANEL_TYPE("usage",            cc_usage_panel_get_type,                NULL),
   PANEL_TYPE("user-accounts",    cc_user_panel_get_type,                 NULL),
 #ifdef BUILD_WACOM
   PANEL_TYPE("wacom",            cc_wacom_panel_get_type,                cc_wacom_panel_static_init_func),
 #endif
-
-
-
+#ifdef BUILD_WWAN
+//  PANEL_TYPE("wwan",             cc_wwan_panel_get_type,                 cc_wwan_panel_static_init_func),
+#endif
 };
 
 /* Override for the panel vtable. When NULL, the default_panels will
@@ -171,8 +159,6 @@ parse_categories (GDesktopAppInfo *app)
     retval = CC_CATEGORY_DEVICES;
   else if (g_strv_contains (const_strv (split), "X-GNOME-DetailsSettings"))
     retval = CC_CATEGORY_DETAILS;
-  else if (g_strv_contains (const_strv (split), "X-GNOME-PrivacySettings"))
-    retval = CC_CATEGORY_PRIVACY;
   else if (g_strv_contains (const_strv (split), "HardwareSettings"))
     retval = CC_CATEGORY_HARDWARE;
 
@@ -216,6 +202,7 @@ ensure_panel_types (void)
 CcPanel *
 cc_panel_loader_load_by_name (CcShell     *shell,
                               const gchar *name,
+                              const gchar *title,
                               GVariant    *parameters)
 {
   GType (*get_type) (void);
@@ -228,6 +215,7 @@ cc_panel_loader_load_by_name (CcShell     *shell,
   return g_object_new (get_type (),
                        "shell", shell,
                        "parameters", parameters,
+                       "title", title,
                        NULL);
 }
 
