@@ -24,16 +24,13 @@
 
 #include <config.h>
 
-#include "cc-debug.h"
+#include "cc-log.h"
 #include "cc-window.h"
 
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
-#include <handy.h>
 #include <string.h>
 #include <time.h>
 
@@ -47,34 +44,29 @@
 
 #define MOUSE_BACK_BUTTON 8
 
-#define DEFAULT_WINDOW_ICON_NAME "regolith-control-center"
+#define DEFAULT_WINDOW_ICON_NAME "gnome-control-center"
 
 struct _CcWindow
 {
-  GtkApplicationWindow parent;
+  AdwApplicationWindow parent;
 
-  GtkRevealer       *back_revealer;
   GtkMessageDialog  *development_warning_dialog;
-  GtkHeaderBar      *header;
-  HdyLeaflet        *header_box;
-  GtkSizeGroup      *header_sizegroup;
-  HdyLeaflet        *main_leaflet;
-  GtkHeaderBar      *panel_headerbar;
+  AdwHeaderBar      *header;
+  AdwNavigationSplitView *split_view;
+  AdwNavigationView *sidebar_view;
+  AdwNavigationPage *main_sidebar_page;
   CcPanelList       *panel_list;
-  GtkButton         *previous_button;
   GtkSearchBar      *search_bar;
   GtkToggleButton   *search_button;
   GtkSearchEntry    *search_entry;
-  GtkBox            *sidebar_box;
-  GtkStack          *stack;
-  GtkBox            *top_left_box;
-  GtkBox            *top_right_box;
+  AdwWindowTitle    *sidebar_title_widget;
 
+  GtkWidget  *old_panel;
   GtkWidget  *current_panel;
   char       *current_panel_id;
   GQueue     *previous_panels;
 
-  GPtrArray  *custom_widgets;
+  GtkWidget  *custom_titlebar;
 
   CcShellModel *store;
 
@@ -86,64 +78,48 @@ struct _CcWindow
 
 static void     cc_shell_iface_init         (CcShellInterface      *iface);
 
-G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, GTK_TYPE_APPLICATION_WINDOW,
+G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, ADW_TYPE_APPLICATION_WINDOW,
                          G_IMPLEMENT_INTERFACE (CC_TYPE_SHELL, cc_shell_iface_init))
 
 enum
 {
   PROP_0,
   PROP_ACTIVE_PANEL,
-  PROP_MODEL
+  PROP_MODEL,
+  PROP_COLLAPSED,
 };
 
 /* Auxiliary methods */
+static void
+load_window_state (CcWindow *self)
+{
+  gint current_width = -1;
+  gint current_height = -1;
+  gboolean maximized = FALSE;
+
+  g_settings_get (self->settings,
+                 "window-state",
+                 "(iib)",
+                 &current_width,
+                 &current_height,
+                 &maximized);
+
+  if (current_width != -1 && current_height != -1)
+    gtk_window_set_default_size (GTK_WINDOW (self), current_width, current_height);
+  if (maximized)
+    gtk_window_maximize (GTK_WINDOW (self));
+}
+
 static gboolean
 in_flatpak_sandbox (void)
 {
   return g_strcmp0 (PROFILE, "development") == 0;
 }
 
-static void
-remove_all_custom_widgets (CcWindow *self)
-{
-  GtkWidget *parent;
-  GtkWidget *widget;
-  guint i;
-
-  CC_ENTRY;
-
-  /* remove from the header */
-  for (i = 0; i < self->custom_widgets->len; i++)
-    {
-      widget = g_ptr_array_index (self->custom_widgets, i);
-      parent = gtk_widget_get_parent (widget);
-
-      g_assert (parent == GTK_WIDGET (self->top_right_box) || parent == GTK_WIDGET (self->top_left_box));
-      gtk_container_remove (GTK_CONTAINER (parent), widget);
-    }
-  g_ptr_array_set_size (self->custom_widgets, 0);
-
-  CC_EXIT;
-}
-
-static void
-show_panel (CcWindow *self)
-{
-  hdy_leaflet_set_visible_child (self->main_leaflet, GTK_WIDGET (self->stack));
-  hdy_leaflet_set_visible_child (self->header_box, GTK_WIDGET (self->panel_headerbar));
-}
-
-static void
-show_sidebar (CcWindow *self)
-{
-  hdy_leaflet_set_visible_child (self->main_leaflet, GTK_WIDGET (self->sidebar_box));
-  hdy_leaflet_set_visible_child (self->header_box, GTK_WIDGET (self->header));
-}
-
 static void
 on_sidebar_activated_cb (CcWindow *self)
 {
-  show_panel (self);
+  adw_navigation_split_view_set_show_content (self->split_view, TRUE);
 }
 
 static gboolean
@@ -155,8 +131,6 @@ activate_panel (CcWindow          *self,
                 CcPanelVisibility  visibility)
 {
   g_autoptr(GTimer) timer = NULL;
-  GtkWidget *sidebar_widget;
-  GtkWidget *title_widget;
   gdouble ellapsed_time;
 
   CC_ENTRY;
@@ -167,40 +141,20 @@ activate_panel (CcWindow          *self,
   if (visibility == CC_PANEL_HIDDEN)
     CC_RETURN (FALSE);
 
-  /* clear any custom widgets */
-  remove_all_custom_widgets (self);
-
   timer = g_timer_new ();
 
-  // Disable setting last panel to avoid issue w/ gnome-control-center
-  // g_settings_set_string (self->settings, "last-panel", id);
-
   /* Begin the profile */
   g_timer_start (timer);
 
   if (self->current_panel)
     g_signal_handlers_disconnect_by_data (self->current_panel, self);
-  self->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, parameters));
+  self->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, name, parameters));
   cc_shell_set_active_panel (CC_SHELL (self), CC_PANEL (self->current_panel));
-  gtk_widget_show (self->current_panel);
 
-  gtk_stack_add_named (self->stack, self->current_panel, id);
+  adw_navigation_split_view_set_content (self->split_view, ADW_NAVIGATION_PAGE (self->current_panel));
 
-  /* switch to the new panel */
-  gtk_widget_show (self->current_panel);
-  gtk_stack_set_visible_child_name (self->stack, id);
-
-  /* set the title of the window */
-  gtk_window_set_role (GTK_WINDOW (self), id);
-  gtk_header_bar_set_title (self->panel_headerbar, name);
-
-  title_widget = cc_panel_get_title_widget (CC_PANEL (self->current_panel));
-  gtk_header_bar_set_custom_title (self->panel_headerbar, title_widget);
-
-  sidebar_widget = cc_panel_get_sidebar_widget (CC_PANEL (self->current_panel));
-  cc_panel_list_add_sidebar_widget (self->panel_list, sidebar_widget);
-  /* Ensure we show the panel when the leaflet is folded and a sidebar widget's
-   * row is activated.
+  /* Ensure we show the panel when the split view is collapsed and a sidebar
+   * widget's row is activated.
    */
   g_signal_connect_object (self->current_panel, "sidebar-activated", G_CALLBACK (on_sidebar_activated_cb), self, G_CONNECT_SWAPPED);
 
@@ -211,6 +165,8 @@ activate_panel (CcWindow          *self,
 
   g_debug ("Time to open panel '%s': %lfs", name, ellapsed_time);
 
+  g_settings_set_string (self->settings, "last-panel", id);
+
   CC_RETURN (TRUE);
 }
 
@@ -258,47 +214,6 @@ find_iter_for_panel_id (CcWindow    *self,
   return valid;
 }
 
-static void
-update_list_title (CcWindow *self)
-{
-  CcPanelListView view;
-  GtkTreeIter iter;
-  g_autofree gchar *title = NULL;
-
-  CC_ENTRY;
-
-  view = cc_panel_list_get_view (self->panel_list);
-  title = NULL;
-
-  switch (view)
-    {
-    case CC_PANEL_LIST_PRIVACY:
-      title = g_strdup (_("Privacy"));
-      break;
-
-    case CC_PANEL_LIST_MAIN:
-      title = g_strdup (_("Settings"));
-      break;
-
-    case CC_PANEL_LIST_WIDGET:
-      find_iter_for_panel_id (self, self->current_panel_id, &iter);
-      gtk_tree_model_get (GTK_TREE_MODEL (self->store),
-                          &iter,
-                          COL_NAME, &title,
-                          -1);
-      break;
-
-    case CC_PANEL_LIST_SEARCH:
-      title = NULL;
-      break;
-    }
-
-  if (title)
-    gtk_header_bar_set_title (self->header, title);
-
-  CC_EXIT;
-}
-
 static void
 on_row_changed_cb (CcWindow     *self,
                    GtkTreePath  *path,
@@ -376,23 +291,6 @@ setup_model (CcWindow *self)
   g_signal_connect_object (model, "row-changed", G_CALLBACK (on_row_changed_cb), self, G_CONNECT_SWAPPED);
 }
 
-static void
-update_headerbar_buttons (CcWindow *self)
-{
-  gboolean is_main_view;
-
-  CC_ENTRY;
-
-  is_main_view = cc_panel_list_get_view (self->panel_list) == CC_PANEL_LIST_MAIN;
-
-  gtk_widget_set_visible (GTK_WIDGET (self->previous_button), !is_main_view);
-  gtk_widget_set_visible (GTK_WIDGET (self->search_button), is_main_view);
-
-  update_list_title (self);
-
-  CC_EXIT;
-}
-
 static gboolean
 set_active_panel_from_id (CcWindow     *self,
                           const gchar  *start_id,
@@ -405,7 +303,6 @@ set_active_panel_from_id (CcWindow     *self,
   g_autofree gchar *name = NULL;
   CcPanelVisibility visibility;
   GtkTreeIter iter;
-  GtkWidget *old_panel;
   CcPanelListView view;
   gboolean activated;
   gboolean found;
@@ -417,9 +314,18 @@ set_active_panel_from_id (CcWindow     *self,
   /* When loading the same panel again, just set its parameters */
   if (g_strcmp0 (self->current_panel_id, start_id) == 0)
     {
+      AdwNavigationPage *sidebar_widget;
+      sidebar_widget = cc_panel_get_sidebar_widget (CC_PANEL (self->current_panel));
+
+      if (sidebar_widget)
+        {
+          adw_navigation_view_push (self->sidebar_view, sidebar_widget);
+          CC_RETURN (TRUE);
+        }
+
       g_object_set (G_OBJECT (self->current_panel), "parameters", parameters, NULL);
       if (force_moving_to_the_panel || self->previous_list_view == view)
-        show_panel (self);
+        adw_navigation_split_view_set_show_content (self->split_view, TRUE);
       self->previous_list_view = view;
       CC_RETURN (TRUE);
     }
@@ -431,7 +337,9 @@ set_active_panel_from_id (CcWindow     *self,
       CC_RETURN (TRUE);
     }
 
-  old_panel = self->current_panel;
+  self->old_panel = self->current_panel;
+  if (self->old_panel)
+    cc_panel_deactivate (CC_PANEL (self->old_panel));
 
   gtk_tree_model_get (GTK_TREE_MODEL (self->store),
                       &iter,
@@ -455,20 +363,15 @@ set_active_panel_from_id (CcWindow     *self,
     add_current_panel_to_history (self, start_id);
 
   if (force_moving_to_the_panel)
-    show_panel (self);
+    adw_navigation_split_view_set_show_content (self->split_view, TRUE);
 
   g_free (self->current_panel_id);
   self->current_panel_id = g_strdup (start_id);
 
   CC_TRACE_MSG ("Current panel id: %s", start_id);
 
-  if (old_panel)
-    gtk_container_remove (GTK_CONTAINER (self->stack), old_panel);
-
   cc_panel_list_set_active_panel (self->panel_list, start_id);
 
-  update_headerbar_buttons (self);
-
   CC_RETURN (TRUE);
 }
 
@@ -512,19 +415,30 @@ switch_to_previous_panel (CcWindow *self)
 }
 
 /* Callbacks */
+
 static void
-on_main_leaflet_folded_changed_cb (CcWindow *self)
+on_split_view_collapsed_changed_cb (CcWindow *self)
 {
   GtkSelectionMode selection_mode;
+  gboolean collapsed;
 
   g_assert (CC_IS_WINDOW (self));
 
-  selection_mode = GTK_SELECTION_SINGLE;
-
-  if (hdy_leaflet_get_folded (self->main_leaflet))
-    selection_mode = GTK_SELECTION_NONE;
+  collapsed = adw_navigation_split_view_get_collapsed (self->split_view);
 
+  selection_mode = collapsed ? GTK_SELECTION_NONE : GTK_SELECTION_SINGLE;
   cc_panel_list_set_selection_mode (self->panel_list, selection_mode);
+
+  if (collapsed && self->current_panel && adw_navigation_view_get_visible_page (self->sidebar_view) == self->main_sidebar_page)
+    {
+      AdwNavigationPage *sidebar_widget;
+      sidebar_widget = cc_panel_get_sidebar_widget (CC_PANEL (self->current_panel));
+
+      if (sidebar_widget)
+        adw_navigation_view_push (self->sidebar_view, sidebar_widget);
+    }
+
+  g_object_notify (G_OBJECT (self), "collapsed");
 }
 
 static void
@@ -542,130 +456,47 @@ search_entry_activate_cb (CcWindow *self)
 {
   gboolean changed;
 
+  if (cc_panel_list_get_view (self->panel_list) != CC_PANEL_LIST_SEARCH)
+    return;
+
   changed = cc_panel_list_activate (self->panel_list);
 
   gtk_search_bar_set_search_mode (self->search_bar, !changed);
 }
 
-static void
-back_button_clicked_cb (CcWindow *self)
+static gboolean
+go_back_shortcut_cb (GtkWidget *widget,
+                     GVariant  *args,
+                     gpointer   user_data)
 {
-  show_sidebar (self);
-}
+  g_debug ("Going to previous panel");
+  switch_to_previous_panel (CC_WINDOW (widget));
 
-static void
-previous_button_clicked_cb (CcWindow *self)
-{
-  g_debug ("Num previous panels? %d", g_queue_get_length (self->previous_panels));
-
-  /* When in search, simply unsed the search mode */
-  if (gtk_search_bar_get_search_mode (self->search_bar))
-    gtk_search_bar_set_search_mode (self->search_bar, FALSE);
-  else
-    cc_panel_list_go_previous (self->panel_list);
-
-  update_headerbar_buttons (self);
-}
-
-static void
-gdk_window_set_cb (CcWindow *self)
-{
-  GdkWindow *window;
-  g_autofree gchar *str = NULL;
-
-  if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
-    return;
-
-  window = gtk_widget_get_window (GTK_WIDGET (self));
-
-  if (!window)
-    return;
-
-  str = g_strdup_printf ("%u", (guint) GDK_WINDOW_XID (window));
-  g_setenv ("GNOME_CONTROL_CENTER_XID", str, TRUE);
+  return GDK_EVENT_STOP;
 }
 
 static gboolean
-window_map_event_cb (CcWindow *self,
-                     GdkEvent *event)
+search_shortcut_cb (GtkWidget *widget,
+                    GVariant  *args,
+                    gpointer   user_data)
 {
-  if (event->any.window == gtk_widget_get_window (GTK_WIDGET (self)))
-    {
-      /* If focus ends up in a category icon view one of the items is
-       * immediately selected which looks odd when we are starting up, so
-       * we  explicitly unset the focus here. */
-      gtk_window_set_focus (GTK_WINDOW (self), NULL);
-    }
-
-  return GDK_EVENT_PROPAGATE;
-}
-
-static gboolean
-window_key_press_event_cb (CcWindow    *self,
-                           GdkEventKey *event)
-{
-  GdkModifierType state;
   CcPanelListView view;
-  GdkKeymap *keymap;
-  gboolean retval;
-  gboolean is_rtl;
+  CcWindow *self;
+  gboolean search;
 
-  retval = GDK_EVENT_PROPAGATE;
-  state = event->state;
-  keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (self)));
-  gdk_keymap_add_virtual_modifiers (keymap, &state);
-
-  state = state & gtk_accelerator_get_default_mod_mask ();
-  is_rtl = gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL;
+  self = CC_WINDOW (widget);
   view = cc_panel_list_get_view (self->panel_list);
 
   /* The search only happens when we're in the MAIN view */
-  if (view == CC_PANEL_LIST_MAIN &&
-      gtk_search_bar_handle_event (self->search_bar, (GdkEvent*) event) == GDK_EVENT_STOP)
-    {
-      return GDK_EVENT_STOP;
-    }
+  if (view != CC_PANEL_LIST_MAIN && view != CC_PANEL_LIST_SEARCH)
+    return GDK_EVENT_PROPAGATE;
 
-  if (state == GDK_CONTROL_MASK)
-    {
-      switch (event->keyval)
-        {
-          case GDK_KEY_s:
-          case GDK_KEY_S:
-          case GDK_KEY_f:
-          case GDK_KEY_F:
-            /* The search only happens when we're in the MAIN view */
-            if (view != CC_PANEL_LIST_MAIN &&
-                view != CC_PANEL_LIST_SEARCH)
-              {
-                break;
-              }
+  search = !gtk_search_bar_get_search_mode (self->search_bar);
+  gtk_search_bar_set_search_mode (self->search_bar, search);
+  if (search)
+    gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
 
-            retval = !gtk_search_bar_get_search_mode (self->search_bar);
-            gtk_search_bar_set_search_mode (self->search_bar, retval);
-            if (retval)
-              gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
-            retval = GDK_EVENT_STOP;
-            break;
-          case GDK_KEY_Q:
-          case GDK_KEY_q:
-          case GDK_KEY_W:
-          case GDK_KEY_w:
-            gtk_widget_destroy (GTK_WIDGET (self));
-            retval = GDK_EVENT_STOP;
-            break;
-        }
-    }
-  else if ((!is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Left) ||
-           (is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Right) ||
-           event->keyval == GDK_KEY_Back)
-    {
-      g_debug ("Going to previous panel");
-      switch_to_previous_panel (self);
-      retval = GDK_EVENT_STOP;
-    }
-
-  return retval;
+  return GDK_EVENT_STOP;
 }
 
 static void
@@ -674,7 +505,7 @@ on_development_warning_dialog_responded_cb (CcWindow *self)
   g_debug ("Disabling development build warning dialog");
   g_settings_set_boolean (self->settings, "show-development-warning", FALSE);
 
-  gtk_widget_hide (GTK_WIDGET (self->development_warning_dialog));
+  gtk_window_close (GTK_WINDOW (self->development_warning_dialog));
 }
 
 /* CcShell implementation */
@@ -683,42 +514,13 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
                                     const gchar  *start_id,
                                     GVariant     *parameters,
                                     GError      **error)
-{
-  return set_active_panel_from_id (CC_WINDOW (shell), start_id, parameters, TRUE, TRUE, error);
-}
-
-static void
-cc_window_embed_widget_in_header (CcShell         *shell,
-                                  GtkWidget       *widget,
-                                  GtkPositionType  position)
 {
   CcWindow *self = CC_WINDOW (shell);
 
-  CC_ENTRY;
+  g_return_val_if_fail (self != NULL, FALSE);
 
-  /* add to header */
-  switch (position)
-    {
-    case GTK_POS_RIGHT:
-      gtk_container_add (GTK_CONTAINER (self->top_right_box), widget);
-      break;
-
-    case GTK_POS_LEFT:
-      gtk_container_add (GTK_CONTAINER (self->top_left_box), widget);
-      break;
-
-    case GTK_POS_TOP:
-    case GTK_POS_BOTTOM:
-    default:
-      g_warning ("Invalid position passed");
-      return;
-    }
-
-  g_ptr_array_add (self->custom_widgets, g_object_ref (widget));
-
-  gtk_size_group_add_widget (self->header_sizegroup, widget);
-
-  CC_EXIT;
+  cc_panel_list_center_activated_row (self->panel_list, TRUE);
+  return set_active_panel_from_id (self, start_id, parameters, TRUE, TRUE, error);
 }
 
 static GtkWidget *
@@ -731,7 +533,6 @@ static void
 cc_shell_iface_init (CcShellInterface *iface)
 {
   iface->set_active_panel_from_id = cc_window_set_active_panel_from_id;
-  iface->embed_widget_in_header = cc_window_embed_widget_in_header;
   iface->get_toplevel = cc_window_get_toplevel;
 }
 
@@ -748,6 +549,27 @@ cc_window_map (GtkWidget *widget)
     gtk_window_present (GTK_WINDOW (self->development_warning_dialog));
 }
 
+static void
+cc_window_unmap (GtkWidget *widget)
+{
+  CcWindow *self = CC_WINDOW (widget);
+  gboolean maximized;
+  gint height;
+  gint width;
+
+  maximized = gtk_window_is_maximized (GTK_WINDOW (self));
+  gtk_window_get_default_size (GTK_WINDOW (self), &width, &height);
+
+  g_settings_set (self->settings,
+                  "window-state",
+                  "(iib)",
+                  width,
+                  height,
+                  maximized);
+
+  GTK_WIDGET_CLASS (cc_window_parent_class)->unmap (widget);
+}
+
 /* GObject Implementation */
 static void
 cc_window_get_property (GObject    *object,
@@ -767,6 +589,10 @@ cc_window_get_property (GObject    *object,
       g_value_set_object (value, self->store);
       break;
 
+    case PROP_COLLAPSED:
+      g_value_set_boolean (value, adw_navigation_split_view_get_collapsed (self->split_view));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -796,32 +622,40 @@ cc_window_set_property (GObject      *object,
     }
 }
 
+static void
+maybe_load_last_panel (CcWindow *self)
+{
+  g_autofree char *id = NULL;
+
+  id = g_settings_get_string (self->settings, "last-panel");
+  if (cc_panel_list_get_current_panel (self->panel_list))
+    return;
+
+  /* select the last used panel, if any, or the first visible panel */
+  if (id != NULL && cc_shell_model_has_panel (self->store, id))
+    {
+      cc_panel_list_center_activated_row (self->panel_list, TRUE);
+      cc_panel_list_set_active_panel (self->panel_list, id);
+    }
+  else
+    cc_panel_list_activate (self->panel_list);
+}
+
 static void
 cc_window_constructed (GObject *object)
 {
   CcWindow *self = CC_WINDOW (object);
-  g_autofree char *id = NULL;
+
+  load_window_state (self);
 
   /* Add the panels */
   setup_model (self);
 
   /* After everything is loaded, select the last used panel, if any,
-   * or the first visible panel */
-  // KG disable selection of last panel to avoid incompat w/ gnome-control-center
-  /*
-  id = g_settings_get_string (self->settings, "last-panel");
-  if (id != NULL && cc_shell_model_has_panel (self->store, id))
-    cc_panel_list_set_active_panel (self->panel_list, id);
-  else */
-  cc_panel_list_activate (self->panel_list);
-
-  g_signal_connect_swapped (self->panel_list,
-                            "notify::view",
-                            G_CALLBACK (update_headerbar_buttons),
-                            self);
-
-  update_headerbar_buttons (self);
-  show_sidebar (self);
+   * or the first visible panel. We do that in an idle handler so we
+   * have a chance to skip it when another panel has been explicitly
+   * activated from commandline parameter or from DBus method */
+  g_idle_add_once ((GSourceOnceFunc) maybe_load_last_panel, self);
 
   G_OBJECT_CLASS (cc_window_parent_class)->constructed (object);
 }
@@ -832,7 +666,6 @@ cc_window_dispose (GObject *object)
   CcWindow *self = CC_WINDOW (object);
 
   g_clear_pointer (&self->current_panel_id, g_free);
-  g_clear_pointer (&self->custom_widgets, g_ptr_array_unref);
   g_clear_object (&self->store);
   g_clear_object (&self->active_panel);
 
@@ -855,6 +688,29 @@ cc_window_finalize (GObject *object)
   G_OBJECT_CLASS (cc_window_parent_class)->finalize (object);
 }
 
+static gboolean
+search_entry_key_pressed_cb (CcWindow              *self,
+                             guint                  keyval,
+                             guint                  keycode,
+                             GdkModifierType        state,
+                             GtkEventControllerKey *key_controller)
+{
+  GtkWidget *toplevel;
+
+  /* When pressing Arrow Down on the entry we move focus to match results list */
+  if (keyval == GDK_KEY_Down || keyval == GDK_KEY_KP_Down)
+    {
+      toplevel = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (self)));
+
+      if (!toplevel)
+        return FALSE;
+
+      return gtk_widget_child_focus (toplevel, GTK_DIR_TAB_FORWARD);
+    }
+
+  return FALSE;
+}
+
 static void
 cc_window_class_init (CcWindowClass *klass)
 {
@@ -868,6 +724,7 @@ cc_window_class_init (CcWindowClass *klass)
   object_class->finalize = cc_window_finalize;
 
   widget_class->map = cc_window_map;
+  widget_class->unmap = cc_window_unmap;
 
   g_object_class_override_property (object_class, PROP_ACTIVE_PANEL, "active-panel");
 
@@ -879,35 +736,40 @@ cc_window_class_init (CcWindowClass *klass)
                                                         CC_TYPE_SHELL_MODEL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/ControlCenter/gtk/cc-window.ui");
+  g_object_class_install_property (object_class,
+                                   PROP_COLLAPSED,
+                                   g_param_spec_boolean ("collapsed",
+                                                         "Collapsed",
+                                                         "Whether the window is collapsed",
+                                                         FALSE,
+                                                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/regolith/Settings/gtk/cc-window.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, back_revealer);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, development_warning_dialog);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, header);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, header_box);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, header_sizegroup);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, main_leaflet);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, panel_headerbar);
+  gtk_widget_class_bind_template_child (widget_class, CcWindow, split_view);
+  gtk_widget_class_bind_template_child (widget_class, CcWindow, sidebar_view);
+  gtk_widget_class_bind_template_child (widget_class, CcWindow, main_sidebar_page);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, panel_list);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, previous_button);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_bar);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_button);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_entry);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, sidebar_box);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, stack);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, top_left_box);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, top_right_box);
+  gtk_widget_class_bind_template_child (widget_class, CcWindow, sidebar_title_widget);
 
-  gtk_widget_class_bind_template_callback (widget_class, back_button_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, gdk_window_set_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_main_leaflet_folded_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_split_view_collapsed_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_development_warning_dialog_responded_cb);
-  gtk_widget_class_bind_template_callback (widget_class, previous_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_panel_cb);
-  gtk_widget_class_bind_template_callback (widget_class, update_list_title);
-  gtk_widget_class_bind_template_callback (widget_class, window_key_press_event_cb);
-  gtk_widget_class_bind_template_callback (widget_class, window_map_event_cb);
+  gtk_widget_class_bind_template_callback (widget_class, search_entry_key_pressed_cb);
+
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_Left, GDK_ALT_MASK, go_back_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_f, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_F, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_q, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Q, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_W, GDK_CONTROL_MASK, "window.close", NULL);
 
   g_type_ensure (CC_TYPE_PANEL_LIST);
 }
@@ -917,16 +779,15 @@ cc_window_init (CcWindow *self)
 {
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  gtk_widget_add_events (GTK_WIDGET (self), GDK_BUTTON_RELEASE_MASK);
-
-  self->settings = g_settings_new ("org.gnome.ControlCenter");
-  self->custom_widgets = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+  self->settings = g_settings_new ("org.regolith.Settings");
   self->previous_panels = g_queue_new ();
   self->previous_list_view = cc_panel_list_get_view (self->panel_list);
 
   /* Add a custom CSS class on development builds */
   if (in_flatpak_sandbox ())
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)), "devel");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "devel");
+
+  gtk_search_bar_set_key_capture_widget (self->search_bar, GTK_WIDGET (self));
 }
 
 CcWindow *
@@ -940,7 +801,6 @@ cc_window_new (GtkApplication *application,
                        "resizable", TRUE,
                        "title", _("Settings"),
                        "icon-name", DEFAULT_WINDOW_ICON_NAME,
-                       "window-position", GTK_WIN_POS_CENTER,
                        "show-menubar", FALSE,
                        "model", model,
                        NULL);
@@ -951,6 +811,6 @@ cc_window_set_search_item (CcWindow   *center,
                            const char *search)
 {
   gtk_search_bar_set_search_mode (center->search_bar, TRUE);
-  gtk_entry_set_text (GTK_ENTRY (center->search_entry), search);
+  gtk_editable_set_text (GTK_EDITABLE (center->search_entry), search);
   gtk_editable_set_position (GTK_EDITABLE (center->search_entry), -1);
 }
